
{*******************************************************}
{                                                       }
{       WDosX Delphi Run-time Library                   }
{       WDOSX TCP/IP ABI Interface Unit                 }
{                                                       }
{       for Delphi 4, 5 WDosX DOS-Extender              }
{       Copyright (c) 2000 by Immo Wache                }
{       e-mail: immo.wache@t-online.de                  }
{                                                       }
{*******************************************************}

{
  Version history:
    IWA 11/20/00 Version 1.1
}

unit WDosAbiDrvr;

interface

uses
  WDosSocketUtils, WDosAbiUtils, WDosResStrings;

{ ABI }
function AbiInit(var AddFlags: Byte; var DriverName: string;
    var DriverInfo: TDriverInfo): Boolean;
function AbiUnload: Integer;

{ TCP }
function TcpOpen(Mode: TTcpMode; SrcePort, RemotePort, TimeOut: Word;
    RemoteIp: TIpAddr; var LocalPort: Word; var Socket: TSocket): Integer;
function TcpClose(Socket: TSocket; SessionAbort: Boolean;
    TimeOut: Word): Integer;
function TcpGet(Socket: TSocket; ModeFlags: TTcpFlags; var Buffer; Len: Word;
    TimeOut: Word; var Transferred: Word; var ReturnFlags: TTcpFlags): Integer;
function TcpPut(Socket: TSocket; ModeFlags: TTcpFlags; TimeOut: Word;
    var Buffer; Len: Word; var Transferred: Word): Integer;
function TcpStatus(Socket: TSocket; var InBytes, OutBytes: Integer;
    var State: Byte; var SessionInfo: TSessionInfo): Integer;

{ UDP }
function UdpOpen(SrcePort, RemotePort: Word; RemoteIp: TIpAddr;
    var LocalPort: Word; var Socket: TSocket): Integer;
function UdpClose(Socket: TSocket): Integer;
function UdpReceive(Socket: TSocket; var Buffer; Len: Word; TimeOut: Word;
    var Transferred, TtlTos, Id: Word): Integer;
function UdpSend(Socket: TSocket; var Buffer; Len, TtlTos, Id: Word;
    var Transferred: Word): Integer;
function UdpStatus(Socket: TSocket; var Packets, NextSize: Integer;
    var SessionInfo: TSessionInfo): Integer;

{ IP }
function IpOpen(Protocol: Byte; RemoteIp: TIpAddr; var Socket: TSocket): Integer;
function IpClose(Socket: TSocket): Integer;
function IpReceive(Socket: TSocket; var Buffer; Len: Word; TimeOut: Word;
    var Transferred, TtlTos, Id: Word): Integer;
function IpSend(Socket: TSocket; var Buffer; Len, TtlTos, Id: Word;
    var Transferred: Word): Integer;
function IpStatus(Socket: TSocket; var Packets, NextSize: Integer;
    var SessionInfo: TSessionInfo): Integer;

implementation

uses
  WDosSysUtils, WDosDpmi;

const
  AbiDpmiError     =8;
  FirstInterrupt   =$60;
  LastInterrupt    =$7F;
  AbiSigOfs        =3;
  AbiSigLen        =8;
  AbiSignature     ='TCP_DRVR';

var
  DosSegment: Integer;         { Realmode-Segment des DOS-Zwischenpuffers }
  DosSelector: DpmiDescriptor; { Selector auf DOS-Zwischenpuffer }
  AbiVector: Integer;
  DataSelector: DpmiDescriptor;

function AbiInfo(Vector: Byte; var AddFlags: Byte; var DriverName: string;
    var DriverInfo: TDriverInfo): Boolean;
var
  Regs: DpmiRmRegisters;
  DSSel: DpmiDescriptor;
  ESSel: DpmiDescriptor;
  Buffer: array[0..255] of Char;
begin
  AbiInfo :=False;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AX :=$00FF;
    if DpmiRealModeInt(Vector, Regs) =DPMIError then Exit;
    if AL =0 then
    begin
      AddFlags :=DH;
      if DpmiRmSelector(DS, DSSel) =DPMIError then Exit;
      if DpmiRmSelector(ES, ESSel) =DPMIError then Exit;
      MoveFar(Ptr(SI)^, DSSel, Buffer, DataSelector, 256);
      DriverName :=Buffer;
      MoveFar(Ptr(DI)^, ESSel, DriverInfo, DataSelector, SizeOf(DriverInfo));
      AbiInfo :=True;
    end;
  end;
end;

function AbiInit(var AddFlags: Byte; var DriverName: string; var DriverInfo:
    TDriverInfo): Boolean;
var
  Vector: RealModePtr;
  IntNum: Byte;
  Signature: array[0..7] of Char;
  Selector: DpmiDescriptor;
begin
  Result :=False;
  for IntNum :=FirstInterrupt to LastInterrupt do
  begin
    if DpmiGetRmVector(IntNum, Vector) =DPMIError then Exit;
    with Vector do
      if (RmOffset =0) and (RmSegment =0) then Continue;
    if DpmiRmSelector(Vector.RmSegment, Selector) =DPMIError then Exit;
    MoveFar(Ptr(Vector.RmOffset +AbiSigOfs)^, Selector, Signature,
        DataSelector, AbiSigLen);
    if StrLComp(AbiSignature, Signature, AbiSigLen) =0 then
    begin
      if AbiInfo(IntNum, AddFlags, DriverName, DriverInfo) then
      begin
        AbiVector :=IntNum;
        Result :=True;
        Exit;
      end;
    end;
  end;
end;

function AbiUnload: Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$01;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    AbiVector :=0;
    Result :=DL;
  end;
end;

{ TCP }

function TcpOpen(Mode: TTcpMode; SrcePort, RemotePort, TimeOut: Word;
    RemoteIp: TIpAddr; var LocalPort: Word; var Socket: TSocket): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$10;
    AL :=Byte(Mode);
    BX :=SrcePort;
    CX :=RemotePort;
    DX :=TimeOut;
    SI :=LongRec(RemoteIp).Hi;
    DI :=LongRec(RemoteIp).Lo;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    LocalPort :=AX;
    Socket :=BX;
    Result :=DL;
  end;
end;

function TcpClose(Socket: TSocket; SessionAbort: Boolean;
    TimeOut: Word): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$11;
    if SessionAbort then AL :=$01 else AL :=$00;
    BX :=Socket;
    DX :=TimeOut;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Result :=DL;
  end;
end;

function TcpGet(Socket: TSocket; ModeFlags: TTcpFlags; var Buffer; Len: Word;
    TimeOut: Word; var Transferred: Word; var ReturnFlags: TTcpFlags): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$12;
    AL :=Byte(ModeFlags);
    BX :=Socket;
    ES :=DosSegment;
    DI :=$00;
    CX :=Len;
    DX :=TimeOut;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Transferred :=AX;
    Result :=DL;
    ReturnFlags :=TTcpFlags(DH);
    if Result =0 then
      MoveFar(Ptr(0)^, DosSelector, Buffer, DataSelector, Transferred);
  end;
end;

function TcpPut(Socket: TSocket; ModeFlags: TTcpFlags; TimeOut: Word;
    var Buffer; Len: Word; var Transferred: Word): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$13;
    AL :=Byte(ModeFlags);
    BX :=Socket;
    ES :=DosSegment;
    DI :=$00;
    DX :=TimeOut;
    CX :=Len;
    BP :=$00;  { TTL & TOS ? }
    MoveFar(Buffer, DataSelector, Ptr(0)^, DosSelector, Len);
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Transferred :=AX;
    Result :=DL;
  end;
end;

function TcpStatus(Socket: TSocket; var InBytes, OutBytes: Integer;
    var State: Byte; var SessionInfo: TSessionInfo): Integer;
var
  Regs: DpmiRmRegisters;
  ESSel: DpmiDescriptor;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$14;
    AL :=$00;
    BX :=Socket;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    if DpmiRmSelector(ES, ESSel) =DPMIError then Exit;
    MoveFar(Ptr(DI)^, ESSel, SessionInfo, DataSelector, SizeOf(SessionInfo));
    with SessionInfo do
    begin
      PortLoc :=Swap(PortLoc);
      PortRem :=Swap(PortRem);
    end;
    State :=DH;
    InBytes :=AX;
    OutBytes :=CX;
    Result :=DL;
  end;
end;

{ UDP }

function UdpOpen(SrcePort, RemotePort: Word; RemoteIp: TIpAddr;
    var LocalPort: Word; var Socket: TSocket): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$20;
    AL :=$00;
    BX :=SrcePort;
    CX :=RemotePort;
    SI :=LongRec(RemoteIp).Hi;
    DI :=LongRec(RemoteIp).Lo;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    LocalPort :=AX;
    Socket :=BX;
    Result :=DL;
  end;
end;

function UdpClose(Socket: TSocket): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$21;
    AL :=$00;
    BX :=Socket;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Result :=DL;
  end;
end;

function UdpReceive(Socket: TSocket; var Buffer; Len: Word; TimeOut: Word;
    var Transferred, TtlTos, Id: Word): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$22;
    AL :=$00;
    BX :=Socket;
    ES :=DosSegment;
    DI :=$00;
    CX :=Len;
    DX :=TimeOut;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Transferred :=AX;
    TtlTos :=BP;
    Id :=SI;
    Result :=DL;
    if Result =0 then
      MoveFar(Ptr(0)^, DosSelector, Buffer, DataSelector, Transferred);
  end;
end;

{:
  Transferred wird aus CX bernommen und entspricht damit immer Len
  AX bleibt ebenfalls unverndert
  Maximale Paketlnge Len betrgt 1472 Byte; andernfalls erscheint
  teBadSession, teilweise strzt hier das Programm auch ab
}
function UdpSend(Socket: TSocket; var Buffer; Len, TtlTos, Id: Word;
    var Transferred: Word): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$23;
    AL :=$00;
    BX :=Socket;
    ES :=DosSegment;
    DI :=$00;
    CX :=Len;
    BP :=TtlTos;
    SI :=Id;
    MoveFar(Buffer, DataSelector, Ptr(0)^, DosSelector, Len);
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Transferred :=CX;
    Result :=DL;
  end;
end;

{:
  NextSize liefert immer 0 zurck :-(
  SessionInfo.Protocol liefert 0
  SessionInfo.Active liefert immer 1
}
function UdpStatus(Socket: TSocket; var Packets, NextSize: Integer;
    var SessionInfo: TSessionInfo): Integer;
var
  Regs: DpmiRmRegisters;
  ESSel: DpmiDescriptor;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$24;
    AL :=$00;
    BX :=Socket;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Packets :=AX;
    NextSize :=CX;
    if DpmiRmSelector(ES, ESSel) =DPMIError then Exit;
    MoveFar(Ptr(DI)^, ESSel, SessionInfo, DataSelector, SizeOf(SessionInfo));
    with SessionInfo do
    begin
      PortLoc :=Swap(PortLoc);
      PortRem :=Swap(PortRem);
    end;
    Result :=DL;
  end;
end;

{ IP }
function IpOpen(Protocol: Byte; RemoteIp: TIpAddr; var Socket: TSocket): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$30;
    AL :=$00;
    BL :=Protocol;
    SI :=LongRec(RemoteIp).Hi;
    DI :=LongRec(RemoteIp).Lo;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Socket :=BX;
    Result :=DL;
  end;
end;

function IpClose(Socket: TSocket): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$31;
    AL :=$00;
    BX :=Socket;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Result :=DL;
  end;
end;

function IpReceive(Socket: TSocket; var Buffer; Len: Word; TimeOut: Word;
    var Transferred, TtlTos, Id: Word): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$32;
    AL :=$00;
    BX :=Socket;
    ES :=DosSegment;
    DI :=$00;
    CX :=Len;
    DX :=TimeOut;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Transferred :=AX;
    Result :=DL;
    if Result =0 then
      MoveFar(Ptr(0)^, DosSelector, Buffer, DataSelector, Transferred);
  end;
end;

function IpSend(Socket: TSocket; var Buffer; Len, TtlTos, Id: Word;
    var Transferred: Word): Integer;
var
  Regs: DpmiRmRegisters;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$33;
    AL :=$00;
    BX :=Socket;
    ES :=DosSegment;
    DI :=$00;
    CX :=Len;
    BP :=TtlTos;
    SI :=Id;
    MoveFar(Buffer, DataSelector, Ptr(0)^, DosSelector, Len);
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Transferred :=CX; { oder AX? }
    Result :=DL;
  end;
end;

{:
  NextSize liefert immer 0 zurck :-(
  SessionInfo.Protocol liefert das aktuelle Protokoll
  SessionInfo.Active liefert immer 1
  PortLoc und PortRem liefern 0 zurck
}
function IpStatus(Socket: TSocket; var Packets, NextSize: Integer;
    var SessionInfo: TSessionInfo): Integer;
var
  Regs: DpmiRmRegisters;
  ESSel: DpmiDescriptor;
begin
  Result :=AbiDpmiError;
  FillChar(Regs, SizeOf(Regs), 0);
  with Regs do
  begin
    AH :=$34;
    AL :=$00;
    BX :=Socket;
    if DpmiRealModeInt(AbiVector, Regs) =DPMIError then Exit;
    Packets :=AX;
    NextSize :=CX;
    if DpmiRmSelector(ES, ESSel) =DPMIError then Exit;
    MoveFar(Ptr(DI)^, ESSel, SessionInfo, DataSelector, SizeOf(SessionInfo));
    Result :=DL;
  end;
end;

initialization
  if DpmiAllocDOSMem(512, DosSegment, DosSelector) =DPMIError then
  begin
    Writeln(SAllocDosMemError);
    Halt(1);
  end;
  asm
    mov DataSelector, DS;
  end;

finalization
  if DpmiFreeDOSMem(DosSelector) =DPMIError then
    Writeln(SFreeDosMemError);

end.
